import functools
from .check import should_instrument
from .wreck import maybe_wreck


def chaos(error_threshold: float, result=None, exception=None):
    def chaos_outer(func):
        if not should_instrument(func):
            return func

        @functools.wraps(func)
        def chaos_inner(*args, **kwargs):
            fn_res = func(*args, **kwargs)
            return maybe_wreck(
                fn_res,
                error_threshold=error_threshold,
                result=result,
                exception=exception,
            )

        return chaos_inner

    return chaos_outer



import random


def maybe_wreck(function_result, error_threshold: float, result=None, exception=None):
    if random.random() >= error_threshold:
        return function_result

    # If we have no exception set, return defined result
    if exception is None:
        return result

    # raise exception
    raise exception



import os
import inspect
from functools import cache, lru_cache
from pathlib import Path


@cache
def is_enabled() -> bool:
    return os.environ.get("CHAOS_ENABLED", "false").lower() == "true"


@lru_cache
def is_function_enabled(func) -> bool:
    """ """
    path = Path(inspect.getfile(func)).relative_to(Path.cwd())
    file_path = str(path.parent).replace("/", ".")
    func_path = f"{file_path}.{path.stem}.{func.__name__}"
    excluded = os.environ.get("CHAOS_EXCLUDE", "").split(",")
    for item in excluded:
        if item and item in func_path:
            return False
    return True


def should_instrument(func) -> bool:
    if not is_enabled():
        return False

    return is_function_enabled(func)
